@@ -77,10 +77,12 @@ try:
 except ImportError:
     from StringIO import StringIO
 
+valid_key_chars_re = re.compile('[\x21-\x7e\x80-\xff]+$')
+
 
 #  Original author: Evan Martin of Danga Interactive
 __author__    = "Sean Reifschneider <jafo-memcached@tummy.com>"
-__version__ = "1.47"
+__version__ = "1.53"
 __copyright__ = "Copyright (C) 2003 Danga Interactive"
 #  http://en.wikipedia.org/wiki/Python_Software_Foundation_License
 __license__   = "Python Software Foundation License"
@@ -91,9 +93,15 @@ SERVER_MAX_KEY_LENGTH = 250
 #  after importing this module.
 SERVER_MAX_VALUE_LENGTH = 1024*1024
 
+
 class _Error(Exception):
     pass
 
+
+class _ConnectionDeadError(Exception):
+    pass
+
+
 try:
     # Only exists in Python 2.4+
     from threading import local
@@ -103,6 +111,10 @@ except ImportError:
         pass
 
 
+_DEAD_RETRY = 30  # number of seconds before retrying a dead server.
+_SOCKET_TIMEOUT = 3  #  number of seconds before sockets timeout.
+
+
 class Client(local):
     """
     Object representing a pool of memcache servers.
@@ -147,8 +159,11 @@ class Client(local):
 
     def __init__(self, servers, debug=0, pickleProtocol=0,
                  pickler=pickle.Pickler, unpickler=pickle.Unpickler,
-                 pload=None, pid=None, server_max_key_length=SERVER_MAX_KEY_LENGTH,
-                 server_max_value_length=SERVER_MAX_VALUE_LENGTH):
+                 pload=None, pid=None,
+                 server_max_key_length=SERVER_MAX_KEY_LENGTH,
+                 server_max_value_length=SERVER_MAX_VALUE_LENGTH,
+                 dead_retry=_DEAD_RETRY, socket_timeout=_SOCKET_TIMEOUT,
+                 cache_cas = False, flush_on_reconnect=0, check_keys=True):
         """
         Create a new Client object with the given list of servers.
 
@@ -162,12 +177,38 @@ class Client(local):
         Useful for cPickle since subclassing isn't allowed.
         @param pid: optional persistent_id function to call on pickle storing.
         Useful for cPickle since subclassing isn't allowed.
+        @param dead_retry: number of seconds before retrying a blacklisted
+        server. Default to 30 s.
+        @param socket_timeout: timeout in seconds for all calls to a server. Defaults
+        to 3 seconds.
+        @param cache_cas: (default False) If true, cas operations will be
+        cached.  WARNING: This cache is not expired internally, if you have
+        a long-running process you will need to expire it manually via
+        client.reset_cas(), or the cache can grow unlimited.
+        @param server_max_key_length: (default SERVER_MAX_KEY_LENGTH)
+        Data that is larger than this will not be sent to the server.
+        @param server_max_value_length: (default SERVER_MAX_VALUE_LENGTH)
+        Data that is larger than this will not be sent to the server.
+        @param flush_on_reconnect: optional flag which prevents a scenario that
+        can cause stale data to be read: If there's more than one memcached
+        server and the connection to one is interrupted, keys that mapped to
+        that server will get reassigned to another. If the first server comes
+        back, those keys will map to it again. If it still has its data, get()s
+        can read stale data that was overwritten on another server. This flag
+        is off by default for backwards compatibility.
+        @param check_keys: (default True) If True, the key is checked to
+        ensure it is the correct length and composed of the right characters.
         """
         local.__init__(self)
         self.debug = debug
+        self.dead_retry = dead_retry
+        self.socket_timeout = socket_timeout
+        self.flush_on_reconnect = flush_on_reconnect
         self.set_servers(servers)
         self.stats = {}
-        self.cas_ids = {}
+        self.cache_cas = cache_cas
+        self.reset_cas()
+        self.do_check_key = check_keys
 
         # Allow users to modify pickling/unpickling behavior
         self.pickleProtocol = pickleProtocol
@@ -186,6 +227,16 @@ class Client(local):
         except TypeError:
             self.picklerIsKeyword = False
 
+    def reset_cas(self):
+        """
+        Reset the cas cache.  This is only used if the Client() object
+        was created with "cache_cas=True".  If used, this cache does not
+        expire internally, so it can grow unbounded if you do not clear it
+        yourself.
+        """
+        self.cas_ids = {}
+
+
     def set_servers(self, servers):
         """
         Set the pool of servers used by this client.
@@ -196,7 +247,10 @@ class Client(local):
             2. Tuples of the form C{("host:port", weight)}, where C{weight} is
             an integer weight value.
         """
-        self.servers = [_Host(s, self.debug) for s in servers]
+        self.servers = [_Host(s, self.debug, dead_retry=self.dead_retry,
+                socket_timeout=self.socket_timeout,
+                flush_on_reconnect=self.flush_on_reconnect)
+                    for s in servers]
         self._init_buckets()
 
     def get_stats(self, stat_args = None):
@@ -215,6 +269,8 @@ class Client(local):
             if not s.connect(): continue
             if s.family == socket.AF_INET:
                 name = '%s:%s (%s)' % ( s.ip, s.port, s.weight )
+            elif s.family == socket.AF_INET6:
+                name = '[%s]:%s (%s)' % ( s.ip, s.port, s.weight )
             else:
                 name = 'unix:%s (%s)' % ( s.address, s.weight )
             if not stat_args:
@@ -238,6 +294,8 @@ class Client(local):
             if not s.connect(): continue
             if s.family == socket.AF_INET:
                 name = '%s:%s (%s)' % ( s.ip, s.port, s.weight )
+            elif s.family == socket.AF_INET6:
+                name = '[%s]:%s (%s)' % ( s.ip, s.port, s.weight )
             else:
                 name = 'unix:%s (%s)' % ( s.address, s.weight )
             serverData = {}
@@ -257,11 +315,10 @@ class Client(local):
         return data
 
     def flush_all(self):
-        'Expire all data currently in the memcache servers.'
+        """Expire all data in memcache servers that are reachable."""
         for s in self.servers:
             if not s.connect(): continue
-            s.send_cmd('flush_all')
-            s.expect("OK")
+            s.flush()
 
     def debuglog(self, str):
         if self.debug:
@@ -375,15 +432,16 @@ class Client(local):
 
         @return: Nonzero on success.
         @param time: number of seconds any subsequent set / update commands
-        should fail. Defaults to 0 for no delay.
+        should fail. Defaults to None for no delay.
         @rtype: int
         '''
-        self.check_key(key)
+        if self.do_check_key:
+            self.check_key(key)
         server, key = self._get_server(key)
         if not server:
             return 0
         self._statlog('delete')
-        if time != None:
+        if time != None and time != 0:
             cmd = "delete %s %d" % (key, time)
         else:
             cmd = "delete %s" % key
@@ -432,16 +490,17 @@ class Client(local):
         returns 0, not -1.
 
         @param delta: Integer amount to decrement by (should be zero or greater).
-        @return: New value after decrementing.
+        @return: New value after decrementing or None on error.
         @rtype: int
         """
         return self._incrdecr("decr", key, delta)
 
     def _incrdecr(self, cmd, key, delta):
-        self.check_key(key)
+        if self.do_check_key:
+            self.check_key(key)
         server, key = self._get_server(key)
         if not server:
-            return 0
+            return None
         self._statlog(cmd)
         cmd = "%s %s %d" % (cmd, key, delta)
         try:
@@ -563,7 +622,7 @@ class Client(local):
         """
         # Check it just once ...
         key_extra_len=len(key_prefix)
-        if key_prefix:
+        if key_prefix and self.do_check_key:
             self.check_key(key_prefix)
 
         # server (_Host) -> list of unprefixed server keys in mapping
@@ -582,7 +641,8 @@ class Client(local):
                 server, key = self._get_server(key_prefix + str_orig_key)
 
             # Now check to make sure key length is proper ...
-            self.check_key(str_orig_key, key_extra_len=key_extra_len)
+            if self.do_check_key:
+                self.check_key(str_orig_key, key_extra_len=key_extra_len)
 
             if not server:
                 continue
@@ -603,41 +663,53 @@ class Client(local):
         1
 
 
-        This method is recommended over regular L{set} as it lowers the number of
-        total packets flying around your network, reducing total latency, since
-        your app doesn't have to wait for each round-trip of L{set} before sending
-        the next one.
+        This method is recommended over regular L{set} as it lowers the
+        number of total packets flying around your network, reducing
+        total latency, since your app doesn't have to wait for each
+        round-trip of L{set} before sending the next one.
 
         @param mapping: A dict of key/value pairs to set.
-        @param time: Tells memcached the time which this value should expire, either
-        as a delta number of seconds, or an absolute unix time-since-the-epoch
-        value. See the memcached protocol docs section "Storage Commands"
-        for more info on <exptime>. We default to 0 == cache forever.
-        @param key_prefix:  Optional string to prepend to each key when sending to memcache. Allows you to efficiently stuff these keys into a pseudo-namespace in memcache:
-            >>> notset_keys = mc.set_multi({'key1' : 'val1', 'key2' : 'val2'}, key_prefix='subspace_')
+        @param time: Tells memcached the time which this value should
+            expire, either as a delta number of seconds, or an absolute
+            unix time-since-the-epoch value. See the memcached protocol
+            docs section "Storage Commands" for more info on <exptime>. We
+            default to 0 == cache forever.
+        @param key_prefix:  Optional string to prepend to each key when
+            sending to memcache. Allows you to efficiently stuff these
+            keys into a pseudo-namespace in memcache:
+
+            >>> notset_keys = mc.set_multi(
+            ...     {'key1' : 'val1', 'key2' : 'val2'}, key_prefix='subspace_')
             >>> len(notset_keys) == 0
             True
             >>> mc.get_multi(['subspace_key1', 'subspace_key2']) == {'subspace_key1' : 'val1', 'subspace_key2' : 'val2'}
             True
 
-            Causes key 'subspace_key1' and 'subspace_key2' to be set. Useful in conjunction with a higher-level layer which applies namespaces to data in memcache.
-            In this case, the return result would be the list of notset original keys, prefix not applied.
+            Causes key 'subspace_key1' and 'subspace_key2' to be
+            set. Useful in conjunction with a higher-level layer which
+            applies namespaces to data in memcache.  In this case, the
+            return result would be the list of notset original keys,
+            prefix not applied.
 
-        @param min_compress_len: The threshold length to kick in auto-compression
-        of the value using the zlib.compress() routine. If the value being cached is
-        a string, then the length of the string is measured, else if the value is an
-        object, then the length of the pickle result is measured. If the resulting
-        attempt at compression yeilds a larger string than the input, then it is
-        discarded. For backwards compatability, this parameter defaults to 0,
-        indicating don't ever try to compress.
-        @return: List of keys which failed to be stored [ memcache out of memory, etc. ].
+        @param min_compress_len: The threshold length to kick in
+            auto-compression of the value using the zlib.compress()
+            routine. If the value being cached is a string, then
+            the length of the string is measured, else if the value
+            is an object, then the length of the pickle result is
+            measured. If the resulting attempt at compression yeilds
+            a larger string than the input, then it is discarded. For
+            backwards compatability, this parameter defaults to 0,
+            indicating don't ever try to compress.
+        @return: List of keys which failed to be stored [ memcache out of
+           memory, etc. ].
         @rtype: list
 
         '''
 
         self._statlog('set_multi')
 
-        server_keys, prefixed_to_orig_key = self._map_and_prefix_keys(mapping.iterkeys(), key_prefix)
+        server_keys, prefixed_to_orig_key = self._map_and_prefix_keys(
+                mapping.iterkeys(), key_prefix)
 
         # send out all requests on each server before reading anything
         dead_servers = []
@@ -672,8 +744,7 @@ class Client(local):
         for server, keys in server_keys.iteritems():
             try:
                 for key in keys:
-                    line = server.readline()
-                    if line == 'STORED':
+                    if server.readline() == 'STORED':
                         continue
                     else:
                         notstored.append(prefixed_to_orig_key[key]) #un-mangle.
@@ -724,66 +795,100 @@ class Client(local):
 
         #  silently do not store if value length exceeds maximum
         if self.server_max_value_length != 0 and \
-           len(val) >= self.server_max_value_length: return(0)
+           len(val) > self.server_max_value_length: return(0)
 
         return (flags, len(val), val)
 
     def _set(self, cmd, key, val, time, min_compress_len = 0):
-        self.check_key(key)
+        if self.do_check_key:
+            self.check_key(key)
         server, key = self._get_server(key)
         if not server:
             return 0
 
-        self._statlog(cmd)
+        def _unsafe_set():
+            self._statlog(cmd)
 
-        store_info = self._val_to_store_info(val, min_compress_len)
-        if not store_info: return(0)
+            store_info = self._val_to_store_info(val, min_compress_len)
+            if not store_info: return(0)
 
-        if cmd == 'cas':
-            if key not in self.cas_ids:
-                return self._set('set', key, val, time, min_compress_len)
-            fullcmd = "%s %s %d %d %d %d\r\n%s" % (
-                    cmd, key, store_info[0], time, store_info[1],
-                    self.cas_ids[key], store_info[2])
-        else:
-            fullcmd = "%s %s %d %d %d\r\n%s" % (
-                    cmd, key, store_info[0], time, store_info[1], store_info[2])
+            if cmd == 'cas':
+                if key not in self.cas_ids:
+                    return self._set('set', key, val, time, min_compress_len)
+                fullcmd = "%s %s %d %d %d %d\r\n%s" % (
+                        cmd, key, store_info[0], time, store_info[1],
+                        self.cas_ids[key], store_info[2])
+            else:
+                fullcmd = "%s %s %d %d %d\r\n%s" % (
+                        cmd, key, store_info[0], time, store_info[1], store_info[2])
+
+            try:
+                server.send_cmd(fullcmd)
+                return(server.expect("STORED", raise_exception=True)
+                        == "STORED")
+            except socket.error, msg:
+                if isinstance(msg, tuple): msg = msg[1]
+                server.mark_dead(msg)
+            return 0
 
         try:
-            server.send_cmd(fullcmd)
-            return(server.expect("STORED") == "STORED")
-        except socket.error, msg:
-            if isinstance(msg, tuple): msg = msg[1]
-            server.mark_dead(msg)
-        return 0
+            return _unsafe_set()
+        except _ConnectionDeadError:
+            # retry once
+            try:
+                if server._get_socket():
+                    return _unsafe_set()
+            except (_ConnectionDeadError, socket.error), msg:
+                server.mark_dead(msg)
+            return 0
 
     def _get(self, cmd, key):
-        self.check_key(key)
+        if self.do_check_key:
+            self.check_key(key)
         server, key = self._get_server(key)
         if not server:
             return None
 
-        self._statlog(cmd)
+        def _unsafe_get():
+            self._statlog(cmd)
+
+            try:
+                server.send_cmd("%s %s" % (cmd, key))
+                rkey = flags = rlen = cas_id = None
+
+                if cmd == 'gets':
+                    rkey, flags, rlen, cas_id, = self._expect_cas_value(server,
+                            raise_exception=True)
+                    if rkey and self.cache_cas:
+                        self.cas_ids[rkey] = cas_id
+                else:
+                    rkey, flags, rlen, = self._expectvalue(server,
+                            raise_exception=True)
+
+                if not rkey:
+                    return None
+                try:
+                    value = self._recv_value(server, flags, rlen)
+                finally:
+                    server.expect("END", raise_exception=True)
+            except (_Error, socket.error), msg:
+                if isinstance(msg, tuple): msg = msg[1]
+                server.mark_dead(msg)
+                return None
+
+            return value
 
         try:
-            server.send_cmd("%s %s" % (cmd, key))
-            rkey = flags = rlen = cas_id = None
-            if cmd == 'gets':
-                rkey, flags, rlen, cas_id, = self._expect_cas_value(server)
-                if rkey:
-                    self.cas_ids[rkey] = cas_id
-            else:
-                rkey, flags, rlen, = self._expectvalue(server)
-
-            if not rkey:
+            return _unsafe_get()
+        except _ConnectionDeadError:
+            # retry once
+            try:
+                if server.connect():
+                    return _unsafe_get()
                 return None
-            value = self._recv_value(server, flags, rlen)
-            server.expect("END")
-        except (_Error, socket.error), msg:
-            if isinstance(msg, tuple): msg = msg[1]
-            server.mark_dead(msg)
+            except (_ConnectionDeadError, socket.error), msg:
+                server.mark_dead(msg)
             return None
-        return value
 
     def get(self, key):
         '''Retrieves a key from the memcache.
@@ -871,9 +976,9 @@ class Client(local):
                 server.mark_dead(msg)
         return retvals
 
-    def _expect_cas_value(self, server, line=None):
+    def _expect_cas_value(self, server, line=None, raise_exception=False):
         if not line:
-            line = server.readline()
+            line = server.readline(raise_exception)
 
         if line and line[:5] == 'VALUE':
             resp, rkey, flags, len, cas_id = line.split()
@@ -881,9 +986,9 @@ class Client(local):
         else:
             return (None, None, None, None)
 
-    def _expectvalue(self, server, line=None):
+    def _expectvalue(self, server, line=None, raise_exception=False):
         if not line:
-            line = server.readline()
+            line = server.readline(raise_exception)
 
         if line and line[:5] == 'VALUE':
             resp, rkey, flags, len = line.split()
@@ -922,7 +1027,7 @@ class Client(local):
                 val = unpickler.load()
             except Exception, e:
                 self.debuglog('Pickle error: %s\n' % e)
-                val = None
+                return None
         else:
             self.debuglog("unknown flags on get: %x\n" % flags)
 
@@ -952,18 +1057,18 @@ class Client(local):
                 len(key) + key_extra_len > self.server_max_key_length:
                 raise Client.MemcachedKeyLengthError("Key length is > %s"
                          % self.server_max_key_length)
-            for char in key:
-                if ord(char) < 33 or ord(char) == 127:
-                    raise Client.MemcachedKeyCharacterError(
-                            "Control characters not allowed")
-
+            if not valid_key_chars_re.match(key):
+                raise Client.MemcachedKeyCharacterError(
+                        "Control characters not allowed")
 
 class _Host(object):
-    _DEAD_RETRY = 30  # number of seconds before retrying a dead server.
-    _SOCKET_TIMEOUT = 3  #  number of seconds before sockets timeout.
 
-    def __init__(self, host, debug=0):
+    def __init__(self, host, debug=0, dead_retry=_DEAD_RETRY,
+                 socket_timeout=_SOCKET_TIMEOUT, flush_on_reconnect=0):
+        self.dead_retry = dead_retry
+        self.socket_timeout = socket_timeout
         self.debug = debug
+        self.flush_on_reconnect = flush_on_reconnect
         if isinstance(host, tuple):
             host, self.weight = host
         else:
@@ -971,6 +1076,9 @@ class _Host(object):
 
         #  parse the connection string
         m = re.match(r'^(?P<proto>unix):(?P<path>.*)$', host)
+        if not m:
+            m = re.match(r'^(?P<proto>inet6):'
+                    r'\[(?P<host>[^\[\]]+)\](:(?P<port>[0-9]+))?$', host)
         if not m:
             m = re.match(r'^(?P<proto>inet):'
                     r'(?P<host>[^:]+)(:(?P<port>[0-9]+))?$', host)
@@ -982,14 +1090,20 @@ class _Host(object):
         if hostData.get('proto') == 'unix':
             self.family = socket.AF_UNIX
             self.address = hostData['path']
+        elif hostData.get('proto') == 'inet6':
+            self.family = socket.AF_INET6
+            self.ip = hostData['host']
+            self.port = int(hostData.get('port') or 11211)
+            self.address = ( self.ip, self.port )
         else:
             self.family = socket.AF_INET
             self.ip = hostData['host']
-            self.port = int(hostData.get('port', 11211))
+            self.port = int(hostData.get('port') or 11211)
             self.address = ( self.ip, self.port )
 
         self.deaduntil = 0
         self.socket = None
+        self.flush_on_next_connect = 0
 
         self.buffer = ''
 
@@ -1010,7 +1124,9 @@ class _Host(object):
 
     def mark_dead(self, reason):
         self.debuglog("MemCache: %s: %s.  Marking dead." % (self, reason))
-        self.deaduntil = time.time() + _Host._DEAD_RETRY
+        self.deaduntil = time.time() + self.dead_retry
+        if self.flush_on_reconnect:
+            self.flush_on_next_connect = 1
         self.close_socket()
 
     def _get_socket(self):
@@ -1019,7 +1135,7 @@ class _Host(object):
         if self.socket:
             return self.socket
         s = socket.socket(self.family, socket.SOCK_STREAM)
-        if hasattr(s, 'settimeout'): s.settimeout(self._SOCKET_TIMEOUT)
+        if hasattr(s, 'settimeout'): s.settimeout(self.socket_timeout)
         try:
             s.connect(self.address)
         except socket.timeout, msg:
@@ -1031,6 +1147,9 @@ class _Host(object):
             return None
         self.socket = s
         self.buffer = ''
+        if self.flush_on_next_connect:
+            self.flush()
+            self.flush_on_next_connect = 0
         return s
 
     def close_socket(self):
@@ -1045,25 +1164,36 @@ class _Host(object):
         """ cmds already has trailing \r\n's applied """
         self.socket.sendall(cmds)
 
-    def readline(self):
+    def readline(self, raise_exception=False):
+        """Read a line and return it.  If "raise_exception" is set,
+        raise _ConnectionDeadError if the read fails, otherwise return
+        an empty string.
+        """
         buf = self.buffer
-        recv = self.socket.recv
+        if self.socket:
+            recv = self.socket.recv
+        else:
+            recv = lambda bufsize: ''
+
         while True:
             index = buf.find('\r\n')
             if index >= 0:
                 break
             data = recv(4096)
             if not data:
-                self.mark_dead('Connection closed while reading from %s'
-                        % repr(self))
-                self.buffer = ''
-                return ''
+                # connection close, let's kill it and raise
+                self.mark_dead('connection closed in readline()')
+                if raise_exception:
+                    raise _ConnectionDeadError()
+                else:
+                    return ''
+
             buf += data
         self.buffer = buf[index+2:]
         return buf[:index]
 
-    def expect(self, text):
-        line = self.readline()
+    def expect(self, text, raise_exception=False):
+        line = self.readline(raise_exception)
         if line != text:
             self.debuglog("while expecting '%s', got unexpected response '%s'"
                     % (text, line))
@@ -1081,6 +1211,10 @@ class _Host(object):
         self.buffer = buf[rlen:]
         return buf[:rlen]
 
+    def flush(self):
+        self.send_cmd('flush_all')
+        self.expect('OK')
+
     def __str__(self):
         d = ''
         if self.deaduntil:
@@ -1088,6 +1222,8 @@ class _Host(object):
 
         if self.family == socket.AF_INET:
             return "inet:%s:%d%s" % (self.address[0], self.address[1], d)
+        elif self.family == socket.AF_INET6:
+            return "inet6:[%s]:%d%s" % (self.address[0], self.address[1], d)
         else:
             return "unix:%s%s" % (self.address, d)
 
@@ -1117,6 +1253,7 @@ if __name__ == "__main__":
                 return "%s (%s)" % (val, type(val))
             return "%s" % val
         def test_setget(key, val):
+            global failures
             print "Testing set/get {'%s': %s} ..." % (to_s(key), to_s(val)),
             mc.set(key, val)
             newval = mc.get(key)
@@ -1154,6 +1291,19 @@ if __name__ == "__main__":
         print "Testing get_multi ...",
         print mc.get_multi(["a_string", "an_integer"])
 
+        #  removed from the protocol
+        #if test_setget("timed_delete", 'foo'):
+        #    print "Testing timed delete ...",
+        #    if mc.delete("timed_delete", 1):
+        #        print "OK"
+        #    else:
+        #        print "FAIL"; failures = failures + 1
+        #    print "Checking results of timed delete ..."
+        #    if mc.get("timed_delete") == None:
+        #        print "OK"
+        #    else:
+        #        print "FAIL"; failures = failures + 1
+
         print "Testing get(unknown value) ...",
         print to_s(mc.get("unknown_value"))
 
@@ -1195,7 +1345,13 @@ if __name__ == "__main__":
 
         print "Testing using insanely long key...",
         try:
-            x = mc.set('a'*SERVER_MAX_KEY_LENGTH + 'aaaa', 1)
+            x = mc.set('a'*SERVER_MAX_KEY_LENGTH, 1)
+        except Client.MemcachedKeyLengthError, msg:
+            print "FAIL"; failures = failures + 1
+        else:
+            print "OK"
+        try:
+            x = mc.set('a'*SERVER_MAX_KEY_LENGTH + 'a', 1)
         except Client.MemcachedKeyLengthError, msg:
             print "OK"
         else:
@@ -1203,13 +1359,13 @@ if __name__ == "__main__":
 
         print "Testing sending a unicode-string key...",
         try:
-            x = mc.set(u'keyhere', 1)
+            x = mc.set(unicode('keyhere'), 1)
         except Client.MemcachedStringEncodingError, msg:
             print "OK",
         else:
             print "FAIL",; failures = failures + 1
         try:
-            x = mc.set((u'a'*SERVER_MAX_KEY_LENGTH).encode('utf-8'), 1)
+            x = mc.set((unicode('a')*SERVER_MAX_KEY_LENGTH).encode('utf-8'), 1)
         except:
             print "FAIL",; failures = failures + 1
         else:
@@ -1223,7 +1379,8 @@ if __name__ == "__main__":
         else:
             print "FAIL"; failures = failures + 1
 
-        print "Testing using a value larger than the memcached value limit...",
+        print "Testing using a value larger than the memcached value limit..."
+        print 'NOTE: "MemCached: while expecting[...]" is normal...'
         x = mc.set('keyhere', 'a'*SERVER_MAX_VALUE_LENGTH)
         if mc.get('keyhere') == None:
             print "OK",
